'use strict'

// a time-based cache that stores recent records with timestamps

class RecentCache {
  constructor (milliseconds, timeKey) {
    this.milliseconds = milliseconds
    this._cache = []
    this.lastGc = 0
    this.timeKey = timeKey || 'time'
  }

  insert (record) {
    this._cache.push(record)
    if ((Date.now() - this.lastGc) > this.milliseconds) {
      // since we don't schedule gc, don't accumulate too many records without gc
      this.gc()
    }
  }

  gc () {
    this._cache = this._cache.filter((record) => {
      return (Date.now() - record[this.timeKey] < this.milliseconds)
    }).sort((a, b) => {
      return a[this.timeKey] - b[this.timeKey]
    })
  }

  entries () {
    this.gc()
    return Array.apply([], this._cache)
  }
}

module.exports = RecentCache
