const hms = require('./ms-to-english/ms-to-english').hms

/**
 * Format some number of seconds into human-readable format.
 * e.g: "10min 5sec"
 */

function duration (seconds) {
  return hms(seconds * 1000, ['y', 'mo', 'w', 'd', 'h', 'm', 's'])
}

/**
 * Two-mode rounding, depending on scale of input:
 * Rounding to two decimal places for numbers between 0 and 1.
 * Normal integer rounding for numbers with absolute value >= 1.
 *
 * e.g.
 * round(0.0123) => 0.01
 * round(123.0123) => 123
 */

function round (num) {
  if (Math.abs(num) >= 1) return Math.round(num)
  else return Math.round(num * 100) / 100
}

/**
 * Standard date output format.
 * @param  {Dateish} date Must be parseable by new Date(date)
 * @return {String}
 */
function date (date) {
  const d = new Date(date)
  if (isNaN(d.getTime())) {
    return 'Invalid Date'
  }
  const timezone = d.toString().split('(')[1].slice(0, -1)
  return `${d.toLocaleDateString()} ${d.toLocaleTimeString()} ${timezone}`
}

function toMilliseconds (val, units) {
  switch (units) {
    case 'seconds':
      return val * 1000
    case 'minutes':
      return val * 60 * 1000
    case 'hours':
      return val * 60 * 60 * 1000
    default:
      return Number.MAX_VALUE
  }
}

function timerElapsed (start, delay) {
  if (typeof start === 'undefined' || typeof delay === 'undefined') {
    return false
  }

  return start + delay < Date.now()
}

function btoa (string) {
  return new Buffer(string.toString(), 'binary').toString('base64')
}

module.exports = {
  duration,
  round,
  date,
  toMilliseconds,
  timerElapsed,
  btoa
}
