#!/usr/bin/env node
'use strict'

const EventEmitter = require('events').EventEmitter
const server = require('./server')
const storage = require('./storage')
const build = require('./build.json.js')
const config = require('./config')
const bole = require('bole')

const conf = config(process.argv)

const loglevelEnv = process.env.LOGLEVEL
var loglevel = loglevelEnv || 'info'

const debugEnv = process.env.NODE_DEBUG
if (debugEnv != null && debugEnv.indexOf('console') >= 0) {
  loglevel = 'debug'
}

bole.output({
  level: loglevel,
  stream: process.stderr
})

const logger = bole(process.title)('cli')

logger.debug('configuration', conf)

var pollingEmitter = new EventEmitter()

var sb = server.createServer(conf, pollingEmitter)

sb.on('error', (e) => {
  logger.error(e, 'skateboard socket error')
  process.exit(1)
})

storage.createApplicationsStream(conf).on('data', (obj) => {
  obj.version = build.describe
  pollingEmitter.emit('output', obj)
})
.on('error', (error) => {
  logger.error(error, 'error in applications stream')
})

storage.startSummaryData(conf)
