nsolid-storage - the N|Solid Storage Server
================================================================================

For more information on N|Solid, please see:

https://docs.nodesource.com/docs/

Start the nsolid-storage server with the following command:

    ./nsolid-storage.js

The server uses `bole` as it's logger, directed to stdout.  To make the output
more readable for humans, a filter `bole.js` has been provided, which you can
use as follows:

    ./nsolid-storage.js | ./bole.js

The server runs in one of two modes:

* storage server
* relay server

Storage server mode is the default, use the `--relay` option to run in relay
server mode.


storage server mode
--------------------------------------------------------------------------------

When running in storage mode, the default configuration will:

* create an API HTTP server listening on port 4000 on all network interfaces
* open zmq sockets on ports 9001, 9002, 9003 on all network interfaces
* store logs, data, etc in `~/.nsolid-storage/`

The N|Solid Console and N|Solid CLI should be configured to point to the API
HTTP server url.

N|Solid processes should access the zmq sockets by setting the following
environment variables:

    NSOLID_COMMAND=localhost:9001
    NSOLID_DATA=localhost:9002
    NSOLID_BULK=localhost:9003


You can check the version of the API HTTP server by accessing the server's base
URL:

```
$ curl http://localhost:4000

{"version":"x.y.z"}
```


relay mode
--------------------------------------------------------------------------------

To support proxying zmq through firewalls, buffering agent data during
Storage downtime, and scaling concerns, nsolid-storage also supports running
in a "relay" mode. The relay sits between agents and a storage server (or another relay). In this mode nsolid-storage only looks like a storage server
to agents and does not have any storage server functionality.

To use the relay mode, both a `--relay` and a `--config` option must be
provided.

The config must contain three extra `[sockets]` settings pointing to an
upstream storage server (or secondary relay):

```toml
[sockets]

commandRemoteAddr = "tcp://<storage-address>:9001"
dataRemoteAddr = "tcp://<storage-address>:9002"
bulkRemoteAddr = "tcp://<storage-address>:9003"
```

See the default configuration file for information regarding additional
relay mode configuration settings.

--

When in relay mode, it it is possible to live-reload the
configuration via a `HUP` signal. Select configuration properties allow
this functionality, as listed below.

```toml
[sockets]
commandRemoteAddr
dataRemoteAddr
bulkRemoteAddr
publicKey

[relay]
maxBufferSizeMb
cleanupBatchSize
logSizeInterval
```

**Warning!**: If you are reloading to connect to a different Storage server
with separate encryption keys, you **must** update the public key in the config
along with the sockets.
If you do not, **data will be lost as garbage** on the Storage server end.

command line options
================================================================================

    -h --help            - print some help text, then exit
    -v --version         - print the program version, then exit
    -p --config-print    - print the configuration, then exit
    -c --config {file}   - use the specified TOML configuration file
    -r --relay           - run in Relay mode (requires `--config`)

The default config file is located in this directory tree at
`etc/default-config.toml` .  That config file contains additional comments
regarding values that can be set.

In addition, environment variables can be used to override the default
configuration values.  The environment variables names have a 1x1 correspondence
with the configuration properties in the config file.  Use the `-h` option to
list the names of the environment variables that can be used.


pre-reqs
================================================================================

You will need a system install of ØMQ, as noted below:

ØMQ - network messaging library
--------------------------------------------------------------------------------

### Mac

Install zeromq and libsodium via [brew][]:

[brew]: http://brew.sh/

    $ brew install libsodium

    ==> Downloading https://homebrew.bintray.com/bottles/libsodium-1.0.10.yosemite.bottle.tar.gz
    ...
    ==> Pouring libsodium-1.0.10.yosemite.bottle.tar.gz
    🍺  /usr/local/Cellar/libsodium/1.0.10: 67 files, 1.1M


    $ brew install zeromq --with-libsodium

    ==> Installing dependencies for zeromq: pkg-config, libsodium
    ==> Installing zeromq dependency: pkg-config
    ==> Downloading https://homebrew.bintray.com/bottles/pkg-config-0.29.1_1.yosemite.bottle.tar.gz
    ######################################################################## 100.0%
    ==> Pouring pkg-config-0.29.1_1.yosemite.bottle.tar.gz
    🍺  /usr/local/Cellar/pkg-config/0.29.1_1: 10 files, 627.3K
    ==> Installing zeromq dependency: libsodium
    ==> Downloading https://homebrew.bintray.com/bottles/libsodium-1.0.11.yosemite.bottle.tar.gz
    ######################################################################## 100.0%
    ==> Pouring libsodium-1.0.11.yosemite.bottle.tar.gz
    🍺  /usr/local/Cellar/libsodium/1.0.11: 66 files, 1.1M
    ==> Installing zeromq
    ==> Using the sandbox
    ==> Downloading https://github.com/zeromq/zeromq4-1/releases/download/v4.1.5/zeromq-4.1.5.tar.gz
    ==> Downloading from https://github-cloud.s3.amazonaws.com/releases/25196738/c773d7de-3488-11e6-89b5-aab54b04312a.gz?X-A
    ######################################################################## 100.0%
    ==> ./configure --prefix=/usr/local/Cellar/zeromq/4.1.5 --with-libsodium
    ==> make
    ==> make install
    🍺  /usr/local/Cellar/zeromq/4.1.5: 12 files, 1.5M, built in 30 seconds

If you run into an error like the following, it may be you have installed zmq,
but not with libsodium; you will need to `brew uninstall zmq` and reinstall it
(after installing libsodium):

    Error: zmq_curve_keypair operation failed. Method support in libzmq v4+ -with-libsodium.

As of the time of this writing, the zmq version is 4.1.5 .

### Ubuntu

You should already have build tools installed via apt-get.

    sudo apt-get install libtool pkg-config build-essential autoconf automake libzmq-dev

Install libsodium.

    git clone git://github.com/jedisct1/libsodium.git
    cd libsodium/
    ./autogen.sh
    ./configure
    make check
    sudo make install
    sudo ldconfig

Download [ØMQ Current Stable POSIX tarball](http://zeromq.org/intro:get-the-software)

    tar zxvpf zeromq-4.1.4.tar.gz
    cd zeromq-4.1.4/
    ./autogen.sh
    ./configure
    make check
    sudo make install
    sudo ldconfig


Tests
================================================================================

Unit tests
--------------------------------------------------------------------------------
Unit tests test individual, isolated pieces of the system. Most unit tests are
limted to a single module or library. The unit tests are located in `test`. You
can run these tests with the following command.

    npm t
