'use strict'

exports.handleRequest = handleRequest

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsStorage.context
  const qData = req.nsStorage.qData

  const commandParams = {
    command: qData.name,
    args: qData.data,
    id: qData.id
  }

  const requestId = context.commandDispatcher.send(commandParams)

  context.commandDispatcher.once(`end:${requestId}`, () => {
    Logger.warn('Unexpected end of command', commandParams)
    res.status(500).send({ error: { message: 'Unexpected end of command.' } })
    endCommand(context, requestId)
  })

  context.commandDispatcher.once(`error:${requestId}`, err => {
    Logger.error('Error running command', err, commandParams)
    res.status(500).send({ error: { message: err.message } })
    endCommand(context, requestId)
  })

  context.commandDispatcher.once(`data:${requestId}`, data => {
    const agent = context.agentManager.byId(data.agentId)
    const response = {
      result: data.body,
      id: data.agentId,
      app: agent.tags.app,
      hostname: agent.tags.hostname,
      tags: (agent.info && agent.info.tags) ? agent.info.tags : [],
      time: data.time
    }
    res.send(response)
    endCommand(context, requestId)
  })
}

function endCommand (context, requestId) {
  context.commandDispatcher.removeAllListeners(`data:${requestId}`)
  context.commandDispatcher.removeAllListeners(`end:${requestId}`)
  context.commandDispatcher.removeAllListeners(`error:${requestId}`)
}
