'use strict'

exports.handleRequest = handleRequest

const lodash = require('lodash')
const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsStorage.context
  const qData = req.nsStorage.qData

  const commandParams = {
    command: 'packages',
    id: qData.id
  }

  const requestId = context.commandDispatcher.send(commandParams)

  context.commandDispatcher.once(`end:${requestId}`, () => {
    Logger.warn('Unexpected end of command', commandParams)
    res.status(500).send('Unexpected end of command.')
    endCommand(context, requestId)
  })

  context.commandDispatcher.once(`error:${requestId}`, err => {
    Logger.error('Error running command', err, commandParams)
    res.status(500).send(err.message)
    endCommand(context, requestId)
  })

  context.commandDispatcher.once(`data:${requestId}`, data => {
    const responseBody = lodash.cloneDeep(data.body)
    responseBody.id = data.agentId
    responseBody.time = data.time

    res.send(JSON.stringify(responseBody) + '\n')
    endCommand(context, requestId)
  })
}

function endCommand (context, requestId) {
  context.commandDispatcher.removeAllListeners(`data:${requestId}`)
  context.commandDispatcher.removeAllListeners(`end:${requestId}`)
  context.commandDispatcher.removeAllListeners(`error:${requestId}`)
}
