'use strict'

exports.handleRequest = handleRequest

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  Logger.debug(`handleRequest(${JSON.stringify(req.body)})`)
  const context = req.nsStorage.context
  let optOut
  if (req.body.optOut === true || req.body.optOut === 'true') optOut = true
  if (req.body.optOut === false || req.body.optOut === 'false') optOut = false

  function responseHandler (err) {
    if (err) {
      res.status(400)
      res.send({ error: { message: 'Could not update license.' } })
      Logger.error(err, 'Could not update license.')
      return
    }

    res.status(200)
    res.send(context.provider.license)
  }

  if (!req.body.license && optOut === false) {
    return responseHandler(new Error('Valid license key is required if not opting out of connected features.'))
  }

  if (req.body.license && optOut === true) {
    // Don't automatically set optOut if they send a license.
    return responseHandler(new Error('Must opt in in order to change a license key'))
  }

  if (!req.body.license && optOut === true) {
    return context.provider.setOptOut(optOut, responseHandler)
  }

  context.provider.syncLicense({ key: req.body.license.key }, responseHandler)
}
