'use strict'

const processThresholds = require('./thresholds')
const createThresholdActionQueue = require('./action-queue')

class Cache {
  constructor (config, previousCache, context) {
    this.processes = previousCache && previousCache.processes ? previousCache.processes : {}
    this.config = config
    this.previousCache = previousCache || null
    this.context = context || (previousCache ? previousCache.context : null)

    if (!previousCache) {
      this.thresholdActionQueue = createThresholdActionQueue(config)
    } else {
      this.thresholdActionQueue = this.previousCache.thresholdActionQueue
      this.previousCache.previousCache = null
    }
  }

  // add a threshold action to the threshold action queue
  addThresholdAction (action) {
    if (this.previousCache) {
      return this.previousCache.addThresholdAction(action)
    }

    this.thresholdActionQueue.push(action)
  }

  /*
    adds a process to the process cache, processes thresholds
    @param {Process} proc
    @param {rebuildActionQueue} set to true to rebuild the threshold action queue on boot
  */
  addProcess (proc, rebuildActionQueue) {
    // process thresholds
    if (this.config && this.config.thresholds) {
      const thresholds = this.config.thresholds
      if (thresholds && thresholds.app && thresholds.app[proc.app]) {
        processThresholds(proc, this, thresholds.app[proc.app], this.config.context, rebuildActionQueue)
      }
    }

    this.processes[proc.id] = proc
  }

  /*
    gets a process by process id

    @param {String} processid
    @return {Process}
  */
  getProcess (processid) {
    if (this.processes[processid]) {
      return this.processes[processid]
    }
    return null
  }
}

module.exports = Cache
